// ----------------------------------------------------------------------------
/*
 * Copyright (c) 2013 Lessmller Lasertechnik GmbH
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
// ----------------------------------------------------------------------------
#ifndef _BOOTLOADER_MODULE_H
#define _BOOTLOADER_MODULE_H

#include "sdo_module.h"
#include "fancy_console.h"

#define BOOTLOADER_NODE_ID	0x7D
#define BOOTLOADER_SERVER_ID	0x600
#define BOOTLOADER_CLIENT_ID	0x580

#define BOOTLOADER_UNLOCK_CODE	0x5A5A

#define BOOTLOADER_SUB_IDX_00			0x00

#define BOOTLOADER_IDX_DEVICE			0x1000
#define BOOTLOADER_IDX_ERROR			0x1001

#define BOOTLOADER_IDX_IDENTITY				0x1018
#define BOOTLOADER_SUB_IDX_VENDOR_ID		0x01
#define BOOTLOADER_SUB_IDX_PART_ID			0x02
#define BOOTLOADER_SUB_IDX_BOOTLOADER_CODE	0x03

#define BOOTLOADER_IDX_PROG_DATA		0x1F50
#define BOOTLOADER_SUB_IDX_PROG_AREA	0x01

#define BOOTLOADER_IDX_PROG_CTRL		0x1F51
#define BOOTLOADER_SUB_IDX_PROG_CTRL	0x01

#define BOOTLOADER_IDX_UNLOCK			0x5000
#define BOOTLOADER_IDX_MEM_RD_ADDR		0x5010
#define BOOTLOADER_IDX_MEM_RD_LEN		0x5011
#define BOOTLOADER_IDX_RAM_WR_ADDR		0x5015
#define BOOTLOADER_IDX_PREP_SECT_WR		0x5020
#define BOOTLOADER_IDX_ERASE_SECT		0x5030

#define BOOTLOADER_IDX_BLANK_CHECK		0x5040
#define BOOTLOADER_SUB_IDX_CHECK_SECT	0x01
#define BOOTLOADER_SUB_IDX_BLANK_OFFSET	0x02

#define BOOTLOADER_IDX_CPY_RAM_MEM		0x5050
#define BOOTLOADER_SUB_IDX_FLASH_ADDR	0x01
#define BOOTLOADER_SUB_IDX_RAM_ADDR		0x02
#define BOOTLOADER_SUB_IDX_CPY_LEN		0x03
	
#define BOOTLOADER_IDX_CMP_MEM			0x5060
#define BOOTLOADER_SUB_IDX_CMP_ADDR1	0x01
#define BOOTLOADER_SUB_IDX_CMP_ADDR2	0x02
#define BOOTLOADER_SUB_IDX_CMP_LEN		0x03
#define BOOTLOADER_SUB_IDX_CMP_OFFSET	0x04

#define BOOTLOADER_IDX_EXEC_ADDR		0x5070
#define BOOTLOADER_SUB_IDX_EXEC_ADDR	0x01
#define BOOTLOADER_SUB_IDX_EXEC_MODE	0x02

#define BOOTLOADER_IDX_SERIAL			0x5100
#define BOOTLOADER_SUB_IDX_SERIAL_1		0x01
#define BOOTLOADER_SUB_IDX_SERIAL_2		0x02
#define BOOTLOADER_SUB_IDX_SERIAL_3		0x03
#define BOOTLOADER_SUB_IDX_SERIAL_4		0x04

int BOOTLOADER_CMD_unlock(unsigned int unlock_code);
int BOOTLOADER_CMD_erase_sector(unsigned char start_sector, unsigned char end_sector);
int BOOTLOADER_CMD_prepare_sector(unsigned char start_sector, unsigned char end_sector);
int BOOTLOADER_CMD_read_id(unsigned int *id);
int BOOTLOADER_CMD_read_boot_code(unsigned int *code);
int BOOTLOADER_CMD_read_serial(unsigned int *serial);
int BOOTLOADER_CMD_write_ram(unsigned int start_address, unsigned int size, unsigned char *data);
int BOOTLOADER_CMD_read_ram(unsigned int start_address, unsigned int size, unsigned char *data);
int BOOTLOADER_CMD_copy_ram_to_flash(unsigned int destination_address, unsigned int source_address, unsigned int size); // Size must be at least 256 or 512,1024,4069. Destination address should be at a 256 boundary
int BOOTLOADER_CMD_compare(unsigned int flash_address, unsigned int ram_address, unsigned int size); // Size must be multiple of 4, addresses must be greater than 512 (0x200)
int BOOTLOADER_CMD_go(unsigned int start_address);

int BOOTLOADER_CMD_reinvoke_isp(void);

#endif