// ----------------------------------------------------------------------------
/*
 * Copyright (c) 2013 Lessmller Lasertechnik GmbH
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
// ----------------------------------------------------------------------------
#ifndef _CAN_MODULE_
#define _CAN_MODULE_

/* CAN Communication Module
 *
 * This Module provides CAN communication functions for sending and receiving messages.
 * 
 */

#define DEBUG

#include "mcp2515_module.h"

/* Data Structure for a CAN Message */
typedef struct can_message_
{
	unsigned int id;					// Message ID (11-29 bits)
	struct flags {
		unsigned char rtr;				// Remote-Transmit-Request-Frame?
#if SUPPORT_EXTENDED_CANID == 1
		unsigned char ext;				// Extended ID?
		flags(void) : rtr(0), ext(0) {};
		flags(unsigned char rtr_, unsigned char ext_) : rtr(rtr_), ext(ext_) {};
#else
		flags(void) : rtr(0) {};
		flags(unsigned char rtr_) : rtr(rtr_) {};
#endif
	} flags;
	
	unsigned char length;				// Length of Payload
	unsigned char data[8];				// Payload

#if SUPPORT_EXTENDED_CANID == 1
	can_message_(void) : id(0x000), flags(0,0), length(0) {};
	can_message_(unsigned int id_, unsigned char rtr_, unsigned char ext_, unsigned char length_) : id(id_), flags(rtr_,ext_), length(length_){};
	can_message_(unsigned int id_, unsigned char rtr_, unsigned char ext_, unsigned char length_, unsigned char *data_) : id(id_), flags(rtr_,ext_), length(length_)
	{
		for (unsigned char i = 0; i < length; i++)
			data[i] = data_[i];
		for (unsigned char i = length; i < 8; i++)
			data[i] = 0x00;
	};
	can_message_(unsigned int id_, unsigned char rtr_, unsigned char ext_, unsigned char length_, ...) : id(id_), flags(rtr_, ext_), length(length_)
	{
		va_list args;
		va_start(args, length_);
		
		for (unsigned int i = 0; i < length; i++)
			data[i] = (unsigned char) va_arg(args, int);

		va_end(args);
	};
#else
	can_message_(void) : id(0x000), flags.rtr(0), length(0) {};
	can_message_(unsigned int id_, unsigned char rtr_, unsigned char length_) : id(id_), flags(rtr_), length(length_){};
	can_message_(unsigned int id_, unsigned char rtr_, unsigned char length_, unsigned char *data_) : id(id_), flags(rtr_), length(length_)
	{
		for (unsigned char i = 0; i < length; i++)
			data[i] = data_[i];
		for (unsigned char i = length; i < 8; i++)
			data[i] = 0x00;
	};
	can_message_(unsigned int id_, unsigned char rtr_, unsigned char length_, ...) : id(id_), flags(rtr_), length(length_)
	{
		va_list args;
		va_start(args, length_);
		
		for (unsigned int i = 0; i < length; i++)
			data[i] = (unsigned char) va_arg(args, int);

		va_end(args);
	};
#endif
} can_message_;

/* Data Structure for a CAN Filter */
typedef struct
{
#if SUPPORT_EXTENDED_CANID == 1
	unsigned char ext;
#endif
	unsigned int mask;
	unsigned int id;
} can_filter_;

typedef trx_status_ can_status_;

int CAN_init(unsigned int baudrate);
int CAN_close(void);

int CAN_send(unsigned char buffer_num, can_message_ msg);														// Configure ID and Data of designated buffer then send RTS
int CAN_recv(unsigned char buffer_num, can_message_ *msg);														// Read ID and Data from designated buffer
int CAN_check_rx(signed char *buffer_num,  unsigned char *filter_num, unsigned char *ext, unsigned char *rtr);	// Check if RX Buffer has a message
int CAN_check_tx(unsigned char *buffer_num);																	// Check if TX Buffer has a message / is free
int CAN_set_filter(unsigned char filter_num, unsigned char mask_num, can_filter_ *flt);							// Configure a Filter with ID and Mask
int CAN_enable_filter(unsigned char buffer_num, unsigned char filter_num, unsigned char extended);				// Enable a Filter for a RX buffer

int CAN_reset(unsigned int baudrate);

int CAN_clear(void);

int CAN_status(can_status_ *status);																			// Fill the can_status structure with all the interesting bits!

#endif