// ----------------------------------------------------------------------------
/*
 * Copyright (c) 2013 Lessmller Lasertechnik GmbH
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
// ----------------------------------------------------------------------------
#ifndef _ISP_MODULE_H
#define _ISP_MODULE_H

#define DEBUG
#define _CRT_SECURE_NO_WARNINGS

#include <io.h>
#include <fcntl.h>
#include <share.h>
#include <errno.h>
#include <sys/stat.h>

#include "bootloader_module.h"
#include "fancy_console.h"

#define BOOTLOADER_BAUDRATE	100000
#define CAN_BAUDRATE		125000

#define ISP_SEARCH_TIMEOUT 20

#define START_SECTOR		0

#define FLASH_SIZE			0x8000			// Size of the Flash Memory
#define FLASH_SECTOR_SIZE	0x1000

#define FLASH_SECTOR(x)		(FLASH_SECTOR_SIZE * x)

#define RAM_START_ADDRESS	0x10000300

#define LPC11C24_ID	0x1430102B		// Identification Number

int ISP_connect(unsigned int baudrate, unsigned int part_id);
int ISP_disconnect(void);

int ISP_unlock(unsigned int unlock_code);

int ISP_write_firmware(char *filename, unsigned int offset);
int ISP_verify(char *filename);
int ISP_verify_byte(char *filename);


int ISP_erase_chip(void);
int ISP_run(unsigned int start_address);
int ISP_reset(void);

int ISP_reinvoke_isp(void);

int load_bin_file(char *filename, unsigned int *filesize, unsigned char *data);


#endif