// ----------------------------------------------------------------------------
/*
 * Copyright (c) 2013 Lessmller Lasertechnik GmbH
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
// ----------------------------------------------------------------------------

#include <stdio.h>
#include <cstdlib>
//#include <stdlib.h> 
#include <conio.h>
#include <stddef.h>

#define _CRT_SECURE_NO_WARNINGS
//#define DEBUG

unsigned char v_level = 1;
char text[80];

#include "fancy_console.h"
#include "isp_module.h"

int process_args(int argc, char *argv[], char *filename, unsigned int *start_address, unsigned int *offset, unsigned int *baudrate,  unsigned int *unlock_code, unsigned int *part_id, unsigned char *action)
{
	unsigned char arg = 1;
	
	if (argc > 1)
	{
		while (arg < argc)
		{
			if (argv[arg][0] != '-')
				return -1;	// Malformed Argument

			switch (argv[arg][1])
			{
				case 'a' : *action = argv[++arg][0]; break;
				case 's' : sscanf_s(argv[++arg], "%x", start_address);  break;
				case 'u' : sscanf_s(argv[++arg], "%x", unlock_code); break;
				case 'b' : sscanf_s(argv[++arg], "%i", baudrate); break;
				case 'p' : sscanf_s(argv[++arg], "%x", part_id); break;
				case 'o' : sscanf_s(argv[++arg], "%x", offset); break;
				case 'f' : strcpy_s(filename, 1024, argv[++arg]); break;
				case 'v' : v_level = atoi(argv[++arg]); break;
				default  : return -2; break;	// Unknown Argument
			}
			arg++;
		}
		return 0;
	} else
	{
		return -3;
	}
}

int main(int argc, char *argv[])
{
	char filename[1024];
	unsigned int start_address	= 0x00000000;
	unsigned char action		= 0;
	unsigned int baudrate		= BOOTLOADER_BAUDRATE;
	unsigned int unlock_code	= BOOTLOADER_UNLOCK_CODE;
	unsigned int part_id		= LPC11C24_ID;
	unsigned int offset			= 0;
	int result = 0;

	header("CAN Bootloader for LPC11Cxx Microcontrollers");
	
	switch (process_args(argc, argv, filename, &start_address, &offset, &baudrate, &unlock_code, &part_id, &action))
	{
		case -3: title("Filename required!"); exit(0); break;
		case -2: title("Unknown Argument!"); exit(0); break;
		case -1: title("Malformed Argument!"); exit(0); break;
	}
	
	sprintf_s(text, 80, "Connecting to PartID 0x%08X @ %ibaud", part_id, baudrate);
	title(text);

	if ((result = ISP_connect(baudrate, part_id)) == 0)
	{
		if ((result = ISP_unlock(unlock_code)) == 0)
		{
			switch (action)
			{
				case 'p' : result = ISP_write_firmware(filename, offset); break;
				case 'e' : result = ISP_erase_chip(); break;
				case 'g' : result = ISP_run(start_address); break;
				case 'r' : result = ISP_reset(); break;
				case 'b' : result = ISP_reinvoke_isp(); break;

				default  : title("Unknown Action. Doing Nothing."); exit(0); break;
			}
		ISP_disconnect();
		}
	}

	switch (result)
	{
		case  -1: title("CAN Interface could not be opened!"); break;
		case  -2: title("Firmware binary file could not be loaded!"); break;
		case  -3: title("Firmware too big for Flash Memory!"); break; 
		case  -4: title("Reading of Part ID failed after timeout!"); break;
		case  -5: title("Chip to be programmed was not identified as LPC11C24!"); break;
		case  -6: title("Unlock failed!"); break;
		case  -7: title("Sector(s) could not be prepared for erasing!"); break;
		case  -8: title("Sector(s) could not be erased!"); break;
		case  -9: title("Download to RAM failed!"); break;
		case -10: title("Sector(s) could not be prepared for writing!"); break;
		case -11: title("Data could not be copied from RAM to Flash!"); break;
		case -12: title("User Program could not be executed!"); break;
		case -13: title("Not connected to CAN Transceiver!"); break;
		case -14: title("No answer after Reinvoking ISP! (This is good, probably!)"); break;
		case -15: title("Data in Flash and RAM not equal!"); break;
		case -16: title("Data could not be verified correctly!"); break;
		case -17: title("Data could not read!"); break;

		case -21: title("Start Address must be a multiple of 4!"); break;
		case -20: title("CAN Interface could not be closed!"); break;
		case   0: title("Action successfull!"); break;
	}

	return 0;
}


