// ----------------------------------------------------------------------------
/*
 * Copyright (c) 2007 Fabian Greif, Roboterclub Aachen e.V.
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: mcp2515_defs.h 6611 2008-07-14 09:58:16Z fabian $
 */
// ----------------------------------------------------------------------------

#ifndef	MCP2515_DEFS_H
#define	MCP2515_DEFS_H

/*name	SPI Kommandos */

#define SPI_RESET		0xC0	
#define	SPI_READ		0x03	
#define	SPI_READ_RX		0x90	
#define	SPI_WRITE		0x02	
#define	SPI_WRITE_TX	0x40	
#define	SPI_RTS			0x80	
#define SPI_READ_STATUS	0xA0	
#define	SPI_RX_STATUS	0xB0	
#define	SPI_BIT_MODIFY	0x05	


/*******************************************************************
 *                  CAN SPI commands                               *
 *******************************************************************/

#define CAN_RESET       0xC0
#define CAN_READ        0x03
#define CAN_WRITE       0x02
#define CAN_RTS         0x80
#define CAN_RTS_TXB0    0x81
#define CAN_RTS_TXB1    0x82
#define CAN_RTS_TXB2    0x84
#define CAN_RD_STATUS   0xA0
#define CAN_BIT_MODIFY  0x05  
#define CAN_RX_STATUS   0xB0
#define CAN_RD_RX_BUFF  0x90
#define CAN_LOAD_TX     0X40  


/*******************************************************************
 *                   Register Definitions                          *
 *******************************************************************/
#define RXF0SIDH	0x00		// FILTER 0 STANDARD IDENTIFIER HIGH
#define RXF0SIDL	0x01		// FILTER 0 STANDARD IDENTIFIER LOW
#define RXF0EID8	0x02		// FILTER 0 EXTENDED IDENTIFIER HIGH
#define RXF0EID0	0x03		// FILTER 0 EXTENDED IDENTIFIER LOW
#define RXF1SIDH	0x04		// FILTER 1 STANDARD IDENTIFIER HIGH
#define RXF1SIDL	0x05		// FILTER 1 STANDARD IDENTIFIER LOW
#define RXF1EID8	0x06		// FILTER 1 EXTENDED IDENTIFIER HIGH
#define RXF1EID0	0x07		// FILTER 1 EXTENDED IDENTIFIER LOW
#define RXF2SIDH	0x08		// FILTER 2 STANDARD IDENTIFIER HIGH
#define RXF2SIDL	0x09		// FILTER 2 STANDARD IDENTIFIER LOW
#define RXF2EID8	0x0A		// FILTER 2 EXTENDED IDENTIFIER HIGH
#define RXF2EID0	0x0B		// FILTER 2 EXTENDED IDENTIFIER LOW
#define BFPCTRL		0x0C		// RXnBF PIN CONTROL AND STATUS
#define TXRTSCTRL	0x0D		// TXnRTS PIN CONTROL AND STATUS REGISTER
#define CANSTAT		0x0E		// CAN Status Register
#define CANCTRL		0x0F		// CAN CONTROL REGISTER

#define RXF3SIDH	0x10		// FILTER 3 STANDARD IDENTIFIER HIGH
#define RXF3SIDL	0x11		// FILTER 3 STANDARD IDENTIFIER LOW
#define RXF3EID8	0x12		// FILTER 3 EXTENDED IDENTIFIER HIGH
#define RXF3EID0	0x13		// FILTER 3 EXTENDED IDENTIFIER LOW
#define RXF4SIDH	0x14		// FILTER 4 STANDARD IDENTIFIER HIGH
#define RXF4SIDL	0x15		// FILTER 4 STANDARD IDENTIFIER LOW
#define RXF4EID8	0x16		// FILTER 4 EXTENDED IDENTIFIER HIGH
#define RXF4EID0	0x17		// FILTER 4 EXTENDED IDENTIFIER LOW
#define RXF5SIDH	0x18		// FILTER 5 STANDARD IDENTIFIER HIGH
#define RXF5SIDL	0x19		// FILTER 5 STANDARD IDENTIFIER LOW
#define RXF5EID8	0x1A		// FILTER 5 EXTENDED IDENTIFIER HIGH
#define RXF5EID0	0x1B		// FILTER 5 EXTENDED IDENTIFIER LOW
#define TEC			0x1C		// Transmit Error Counter
#define REC         0x1D		// Receive	Error Counter

#define RXM0SIDH	0x20		// MASK 0 STANDARD IDENTIFIER HIGH
#define RXM0SIDL	0x21		// MASK 0 STANDARD IDENTIFIER LOW
#define RXM0EID8	0x22		// MASK 0 EXTENDED IDENTIFIER HIGH
#define RXM0EID0	0x23		// MASK 0 EXTENDED IDENTIFIER LOW
#define RXM1SIDH	0x24		// MASK 1 STANDARD IDENTIFIER HIGH
#define RXM1SIDL	0x25		// MASK 1 STANDARD IDENTIFIER LOW
#define RXM1EID8	0x26		// MASK 1 EXTENDED IDENTIFIER HIGH
#define RXM1EID0	0x27		// MASK 1 EXTENDED IDENTIFIER LOW
#define CNF3		0x28		// CONFIGURATION 1
#define CNF2		0x29		// CONFIGURATION 1
#define CNF1		0x2A		// CONFIGURATION 1
#define CANINTE		0x2B		// INTERRUPT ENABLE
#define CANINTF		0x2C		// INTERRUPT FLAG
#define EFLG		0x2D		// Error Flag

#define TXB0CTRL	0x30		// TRANSMIT BUFFER 0 CONTROL REGISTER
#define TXB0SIDH	0x31		// TRANSMIT BUFFER 0 STANDARD IDENTIFIER HIGH
#define TXB0SIDL	0x32		// TRANSMIT BUFFER 0 STANDARD IDENTIFIER LOW
#define TXB0EID8	0x33		// TRANSMIT BUFFER 0 EXTENDED IDENTIFIER HIGH
#define TXB0EID0	0x34		// TRANSMIT BUFFER 0 EXTENDED IDENTIFIER LOW
#define TXB0DLC		0x35		// TRANSMIT BUFFER 0 DATA LENGTH CODE
#define TXB0D0		0x36		// TRANSMIT BUFFER 0 DATA BYTE 0
#define TXB0D1		0x37		// TRANSMIT BUFFER 0 DATA BYTE 1
#define TXB0D2		0x38		// TRANSMIT BUFFER 0 DATA BYTE 2
#define TXB0D3		0x39		// TRANSMIT BUFFER 0 DATA BYTE 3
#define TXB0D4		0x3A		// TRANSMIT BUFFER 0 DATA BYTE 4
#define TXB0D5		0x3B		// TRANSMIT BUFFER 0 DATA BYTE 5
#define TXB0D6		0x3C		// TRANSMIT BUFFER 0 DATA BYTE 6
#define TXB0D7		0x3D		// TRANSMIT BUFFER 0 DATA BYTE 7

#define TXB1CTRL	0x40		// TRANSMIT BUFFER 1 CONTROL REGISTER
#define TXB1SIDH	0x41		// TRANSMIT BUFFER 1 STANDARD IDENTIFIER HIGH
#define TXB1SIDL	0x42		// TRANSMIT BUFFER 1 STANDARD IDENTIFIER LOW
#define TXB1EID8	0x43		// TRANSMIT BUFFER 1 EXTENDED IDENTIFIER HIGH
#define TXB1EID0	0x44		// TRANSMIT BUFFER 1 EXTENDED IDENTIFIER LOW
#define TXB1DLC		0x45		// TRANSMIT BUFFER 1 DATA LENGTH CODE
#define TXB1D0		0x46		// TRANSMIT BUFFER 1 DATA BYTE 0
#define TXB1D1		0x47		// TRANSMIT BUFFER 1 DATA BYTE 1
#define TXB1D2		0x48		// TRANSMIT BUFFER 1 DATA BYTE 2
#define TXB1D3		0x49		// TRANSMIT BUFFER 1 DATA BYTE 3
#define TXB1D4		0x4A		// TRANSMIT BUFFER 1 DATA BYTE 4
#define TXB1D5		0x4B		// TRANSMIT BUFFER 1 DATA BYTE 5
#define TXB1D6		0x4C		// TRANSMIT BUFFER 1 DATA BYTE 6
#define TXB1D7		0x4D		// TRANSMIT BUFFER 1 DATA BYTE 7

#define TXB2CTRL	0x50		// TRANSMIT BUFFER 2 CONTROL REGISTER
#define TXB2SIDH	0x51		// TRANSMIT BUFFER 2 STANDARD IDENTIFIER HIGH
#define TXB2SIDL	0x52		// TRANSMIT BUFFER 2 STANDARD IDENTIFIER LOW
#define TXB2EID8	0x53		// TRANSMIT BUFFER 2 EXTENDED IDENTIFIER HIGH
#define TXB2EID0	0x54		// TRANSMIT BUFFER 2 EXTENDED IDENTIFIER LOW
#define TXB2DLC		0x55		// TRANSMIT BUFFER 2 DATA LENGTH CODE
#define TXB2D0		0x56		// TRANSMIT BUFFER 2 DATA BYTE 0
#define TXB2D1		0x57		// TRANSMIT BUFFER 2 DATA BYTE 1
#define TXB2D2		0x58		// TRANSMIT BUFFER 2 DATA BYTE 2
#define TXB2D3		0x59		// TRANSMIT BUFFER 2 DATA BYTE 3
#define TXB2D4		0x5A		// TRANSMIT BUFFER 2 DATA BYTE 4
#define TXB2D5		0x5B		// TRANSMIT BUFFER 2 DATA BYTE 5
#define TXB2D6		0x5C		// TRANSMIT BUFFER 2 DATA BYTE 6
#define TXB2D7		0x5D		// TRANSMIT BUFFER 2 DATA BYTE 7

#define RXB0CTRL	0x60		// RECEIVE BUFFER 0 CONTROL
#define RXB0SIDH	0x61		// RECEIVE BUFFER 0 STANDARD IDENTIFIER HIGH
#define RXB0SIDL	0x62		// RECEIVE BUFFER 0 STANDARD IDENTIFIER LOW
#define RXB0EID8	0x63		// RECEIVE BUFFER 0 EXTENDED IDENTIFIER HIGH
#define RXB0EID0	0x64		// RECEIVE BUFFER 0 EXTENDED IDENTIFIER LOW
#define RXB0DLC		0x65		// RECEIVE BUFFER 0 STANDARD IDENTIFIER LOW
#define RXB0D0		0x66		// RECEIVE BUFFER 0 DATA BYTE 0
#define RXB0D1		0x67		// RECEIVE BUFFER 0 DATA BYTE 1
#define RXB0D2		0x68		// RECEIVE BUFFER 0 DATA BYTE 2
#define RXB0D3		0x69		// RECEIVE BUFFER 0 DATA BYTE 3
#define RXB0D4		0x6A		// RECEIVE BUFFER 0 DATA BYTE 4
#define RXB0D5		0x6B		// RECEIVE BUFFER 0 DATA BYTE 5
#define RXB0D6		0x6C		// RECEIVE BUFFER 0 DATA BYTE 6
#define RXB0D7		0x6D		// RECEIVE BUFFER 0 DATA BYTE 7

#define RXB1CTRL	0x70		// RECEIVE BUFFER 1 CONTROL
#define RXB1SIDH	0x71		// RECEIVE BUFFER 1 STANDARD IDENTIFIER HIGH
#define RXB1SIDL	0x72		// RECEIVE BUFFER 1 STANDARD IDENTIFIER LOW
#define RXB1EID8	0x73		// RECEIVE BUFFER 1 EXTENDED IDENTIFIER HIGH
#define RXB1EID0	0x74		// RECEIVE BUFFER 1 EXTENDED IDENTIFIER LOW
#define RXB1DLC		0x75		// RECEIVE BUFFER 1 STANDARD IDENTIFIER LOW
#define RXB1D0		0x76		// RECEIVE BUFFER 1 DATA BYTE 0
#define RXB1D1		0x77		// RECEIVE BUFFER 1 DATA BYTE 1		
#define RXB1D2		0x78		// RECEIVE BUFFER 1 DATA BYTE 2
#define RXB1D3		0x79		// RECEIVE BUFFER 1 DATA BYTE 3
#define RXB1D4		0x7A		// RECEIVE BUFFER 1 DATA BYTE 4
#define RXB1D5		0x7B		// RECEIVE BUFFER 1 DATA BYTE 5
#define RXB1D6		0x7C		// RECEIVE BUFFER 1 DATA BYTE 6
#define RXB1D7		0x7D		// RECEIVE BUFFER 1 DATA BYTE 7


/*******************************************************************
 *          Bitdefinition der verschiedenen Register               *
 *******************************************************************/

/* Bitdefinition von TXRTSCTRL */
#define B2RTS		5		// TX2RTS Pin State bit
#define B1RTS		4		// TX1RTX Pin State bit
#define B0RTS		3		// TX0RTS Pin State bit
#define B2RTSM		2		// TX2RTS Pin mode bit
#define B1RTSM		1		// TX1RTS Pin mode bit
#define B0RTSM		0		// TX0RTS Pin mode bit

/* Bitdefinition von CANSTAT */
#define OPMOD2		7		// Operation mode bits
#define OPMOD1		6
#define OPMOD0		5
#define ICOD2		3		// Interrupt Flag Code bits
#define ICOD1		2
#define ICOD0		1

/* Bitdefinition von CANCTRL */
#define REQOP2		7		// Request Operation mode bits
#define REQOP1		6		// Request Operation mode bits
#define REQOP0		5		// Request Operation mode bits
#define CLKPRE1		1		// CLKOUT Pin Prescaler bits
#define CLKPRE0		0		// CLKOUT Pin Prescaler bits


/* Bitdefinition von CNF3 */
#define SOF			7		// Start-of-Frame signal bit
#define PHSEG22		2		// PS2 Length bits
#define PHSEG21		1
#define PHSEG20		0


/* Bitdefinition von CNF2 */
#define PHSEG12		5		// PS1 Length bits
#define PHSEG11		4
#define PHSEG10		3
#define PRSEG2		2		// Propagation Segment Length bits
#define PRSEG1		1
#define PRSEG0		0

/* Bitdefinition von CNF1 */
#define SJW1		7		// Synchronization Jump Width Length bits
#define SJW0		6
#define BRP5		5		// Baud Rate Prescaler bits
#define BRP4		4
#define BRP3		3
#define BRP2		2
#define BRP1		1
#define BRP0		0


/* Bitdefinition von EFLG */
#define RX1OVR		7		// Receive Buffer 1 Overflow Flag bit
#define RX0OVR		6		// Receive Buffer 0 Overflow Flag bit
#define TXB0		5		// Bus-Off Error Flag bit
#define TXEP		4		// Transmit Error-Passive Flag bit
#define RXEP		3		// Receive Error-Passive Flag bit
#define TXWAR		2		// Transmit Error Warning Flag bit
#define RXWAR		1		// Receive Error Warning Flag bit
#define EWARN		0		// Error Warning Flag bit

/* Bitdefinition von TXBnCTRL (n = 0, 1, 2) */
#define ABTF		6		// Message Aborted Flag bit
#define MLOA		5		// Message Lost Arbitration bit
							// TXP<1:0>: Transmit Buffer Priority bits	
#define TXP1		1		// 11 = Highest Message Priority ,10 = High Intermediate Message Priority
#define TXP0		0		// 01 = Low Intermediate Message Priority, 00 = Lowest Message Priority
							

/* Bitdefinition von RXB0CTRL */
#define RXM1		6		// Receive Buffer Operating mode bits
#define RXM0		5		// Receive Buffer Operating mode bits
#define RXRTR		3		// Received Remote Transfer Request bit
#define BUKT1		1		// Read-only Copy of BUKT bit (used internally by the MCP2515)
#define FILHIT0		0

/* Bitdefinition von TXBnSIDL (n = 0, 1) */
#define	EXIDE		3		// Extended Identifier Enable bit

/* Bitdefinition von RXB1CTRL
 *  RXM1, RXM0, RXRTR und FILHIT0 sind schon fuer RXB0CTRL definiert */
#define FILHIT2		2		// Filter Hit bits - indicates which acceptance filter enabled reception of message
#define FILHIT1		1		// Filter Hit bits - indicates which acceptance filter enabled reception of message

/* Bitdefinition von RXBnSIDL (n = 0, 1) */
#define	SRR			4		// Standard Frame Remote Transmit Request bit (valid only if IDE bit = 0)
#define	IDE			3		// Extended Identifier Flag bit

/* Bitdefinition von RXBnDLC (n = 0, 1) TXBnDLC   (gleiche Bits) */
#define	RTR			6		// Extended Frame Remote Transmission Request bit
#define	DLC3		3		// Data Length Code bits
#define	DLC2		2		// Data Length Code bits
#define	DLC1		1		// Data Length Code bits
#define DLC0		0		// Data Length Code bits


/*******************************************************************
 *               Bit register masks                                *
 *******************************************************************/

/* TXBnCTRL */
#define TXREQ           0x08	// Message Transmit Request bit
#define TXP             0x03	// Transmit Buffer Priority bits (hoechste Prioritaet) 

/* RXBnCTRL */
#define RXM             0x60	// Receive Buffer Operating mode bits // Turn mask/filters off; receive any message
#define BUKT            0x04	// Rollover Enable bit
#define FILHIT25		0x07	// Filter Hit Bits fpr RXF2-RXF5 in RXB1CTRL

/* CANCTRL */
#define REQOP           0xE0	// Request Operation mode bits
#define ABAT            0x10	// Abort All Pending Transmissions bit
#define	OSM         	0x08	// One-Shot mode bit
#define CLKEN           0x04	// CLKOUT Pin Enable bit
#define CLKPRE          0x03	// CLKOUT Pin Prescaler bits

/* CANSTAT */
#define REQOP           0xE0	// Request Operation mode bits
#define ICOD            0x0E	// Interrupt Flag Code bits

/* CANINTE */   
#define RX0IE           0x01	// Receive Buffer 0 Full Interrupt Enable bit
#define RX1IE           0x02	// Receive Buffer 1 Full Interrupt Enable bit
#define TX0IE           0x04	// Transmit Buffer 0 Empty Interrupt Enable bit
#define TX1IE           0x80	// Transmit Buffer 1 Empty Interrupt Flag bit
#define TX2IE           0x10	// Transmit Buffer 2 Empty Interrupt Flag bit
#define ERRIE           0x20	// Error Interrupt Enable bit 
#define WAKIE           0x40	// Wake-up Interrupt Enable bit
#define MERRE           0x80	// Message Error Interrupt Enable bit

/* CANINTF */   
#define RX0IF           0x01	// Receive Buffer 0 Full Interrupt Flag bit
#define RX1IF           0x02	// Receive Buffer 1 Full Interrupt Flag bit
#define TX0IF           0x04	// Transmit Buffer 0 Empty Interrupt Flag bit
#define TX1IF           0x80	// Transmit Buffer 1 Empty Interrupt Flag bit
#define TX2IF           0x10	// Transmit Buffer 2 Empty Interrupt Flag bit
#define ERRIF           0x20	// Error Interrupt Flag bit
#define WAKIF           0x40	// Wake-up Interrupt Flag bit
#define MERRF           0x80	// Message Error Interrupt Flag bit
/* BFPCTRL */
#define B1BFS           0x20	// RX1BF Pin State bit
#define B0BFS           0x10	// RX0BF Pin State bit
#define B1BFE           0x08	// RX1BF Pin Function Enable bit
#define B0BFE           0x04	// RX0BF Pin Function Enable bit
#define B1BFM           0x02	// RX1BF Pin Operation mode bit
#define B0BFM           0x01	// RX0BF Pin Operation mode bit

/* CNF1 Masks */
#define SJW             0xC0	// Synchronization Jump Width Length bits
#define BRP             0x3F	// Baud Rate Prescaler bits

/* CNF2 Masks */
#define BTLMODE         0x80	// PS2 Bit Time Length bit
#define SAM             0x40	// Sample Point Configuration bit
#define PHSEG1          0x38
#define PRSEG           0x07	// Propagation Segment Length bits

/* CNF3 Masks */
#define WAKFIL          0x40	// Wake-up Filter bit
#define PHSEG2          0x07

/* TXRTSCTRL Masks */
#define TXB2RTS         0x04
#define TXB1RTS         0x02
#define TXB0RTS         0x01


/*******************************************************************
 *                    Bit Timing Configuration                     *
 *******************************************************************/
 
/* CNF1 */
#define SJW_1TQ         0x00	// 0x40
#define SJW_2TQ         0x40	// 0x80
#define SJW_3TQ         0x80	// 0x90
#define SJW_4TQ         0xC0

/* CNF2 */
#define BTLMODE_CNF3    0x80
#define BTLMODE_PH1_IPT 0x00

#define SMPL_3X         0x40
#define SMPL_1X         0x00

#define PHSEG1_8TQ      0x38
#define PHSEG1_7TQ      0x30
#define PHSEG1_6TQ      0x28
#define PHSEG1_5TQ      0x20
#define PHSEG1_4TQ      0x18
#define PHSEG1_3TQ      0x10
#define PHSEG1_2TQ      0x08
#define PHSEG1_1TQ      0x00

#define PRSEG_8TQ       0x07
#define PRSEG_7TQ       0x06
#define PRSEG_6TQ       0x05
#define PRSEG_5TQ       0x04
#define PRSEG_4TQ       0x03
#define PRSEG_3TQ       0x02
#define PRSEG_2TQ       0x01
#define PRSEG_1TQ       0x00

/* CNF3 */
#define PHSEG2_8TQ      0x07
#define PHSEG2_7TQ      0x06
#define PHSEG2_6TQ      0x05
#define PHSEG2_5TQ      0x04
#define PHSEG2_4TQ      0x03
#define PHSEG2_3TQ      0x02
#define PHSEG2_2TQ      0x01
#define PHSEG2_1TQ      0x00

#define SOF_ENABLED     0x80
#define SOF_DISABLED    0x00
#define WAKFIL_ENABLED  0x40
#define WAKFIL_DISABLED 0x00    


/*******************************************************************
 *                  Control/Configuration Registers                *
 *******************************************************************/

/* CANINTE */
#define RX0IE_ENABLED   0x01
#define RX0IE_DISABLED  0x00
#define RX1IE_ENABLED   0x02
#define RX1IE_DISABLED  0x00
#define G_RXIE_ENABLED  0x03
#define G_RXIE_DISABLED 0x00

#define TX0IE_ENABLED   0x04
#define TX0IE_DISABLED  0x00
#define TX1IE_ENABLED   0x08
#define TX2IE_DISABLED  0x00
#define TX2IE_ENABLED   0x10
#define TX2IE_DISABLED  0x00
#define G_TXIE_ENABLED  0x1C
#define G_TXIE_DISABLED 0x00

#define ERRIE_ENABLED   0x20
#define ERRIE_DISABLED  0x00
#define WAKIE_ENABLED   0x40
#define WAKIE_DISABLED  0x00
#define IVRE_ENABLED    0x80
#define IVRE_DISABLED   0x00

/* CANINTF */
#define RX0IF_SET       0x01
#define RX0IF_RESET     0x00
#define RX1IF_SET       0x02
#define RX1IF_RESET     0x00
#define TX0IF_SET       0x04
#define TX0IF_RESET     0x00
#define TX1IF_SET       0x08
#define TX2IF_RESET     0x00
#define TX2IF_SET       0x10
#define TX2IF_RESET     0x00
#define ERRIF_SET       0x20
#define ERRIF_RESET     0x00
#define WAKIF_SET       0x40
#define WAKIF_RESET     0x00
#define IVRF_SET        0x80
#define IVRF_RESET      0x00

/* CANCTRL */ 
#define REQOP_CONFIG    0x80
#define REQOP_LISTEN    0x60
#define REQOP_LOOPBACK  0x40
#define REQOP_SLEEP     0x20
#define REQOP_NORMAL    0x00

#define ABORT           0x10

#define OSM_ENABLED     0x08

#define CLKOUT_ENABLED  0x04
#define CLKOUT_DISABLED 0x00
#define CLKOUT_PRE_8    0x03
#define CLKOUT_PRE_4    0x02
#define CLKOUT_PRE_2    0x01
#define CLKOUT_PRE_1    0x00

/* CANSTAT */
#define OPMODE_CONFIG   0x80
#define OPMODE_LISTEN   0x60
#define OPMODE_LOOPBACK 0x40
#define OPMODE_SLEEP    0x20
#define OPMODE_NORMAL   0x00

/* RXBnCTRL */
#define RXM_RCV_ALL     0x60
#define RXM_VALID_EXT   0x40
#define RXM_VALID_STD   0x20
#define RXM_VALID_ALL   0x00

#define RXRTR_REMOTE    0x08
#define RXRTR_NO_REMOTE 0x00

#define BUKT_ROLLOVER    0x04
#define BUKT_NO_ROLLOVER 0x00

#define FILHIT0_FLTR_1  0x01
#define FILHIT0_FLTR_0  0x00

#define FILHIT1_FLTR_5  0x05
#define FILHIT1_FLTR_4  0x04
#define FILHIT1_FLTR_3  0x03
#define FILHIT1_FLTR_2  0x02
#define FILHIT1_FLTR_1  0x01
#define FILHIT1_FLTR_0  0x00


/* TXBnCTRL */
#define TXREQ_SET       0x08
#define TXREQ_CLEAR     0x00

#define TXP_HIGHEST     0x03
#define TXP_INTER_HIGH  0x02
#define TXP_INTER_LOW   0x01
#define TXP_LOWEST      0x00
    

/*******************************************************************
 *                  Register Bit Masks                             *
 *******************************************************************/
 
 #define DLC_0          0x00
 #define DLC_1          0x01
 #define DLC_2          0x02
 #define DLC_3          0x03
 #define DLC_4          0x04
 #define DLC_5          0x05
 #define DLC_6          0x06
 #define DLC_7          0x07    
 #define DLC_8          0x08
 

/*******************************************************************
 *                  CAN SPI commands                               *
 *******************************************************************/

#define CAN_RESET       0xC0
#define CAN_READ        0x03
#define CAN_WRITE       0x02
#define CAN_RTS         0x80
#define CAN_RD_STATUS   0xA0
#define CAN_BIT_MODIFY  0x05  
#define CAN_RX_STATUS   0xB0
#define CAN_RD_RX_BUF	0x90
#define CAN_LD_TX_BUF   0X40  


/*******************************************************************
 *                  Miscellaneous                                  *
 *******************************************************************/
/*
#define DUMMY_BYTE      0x00
#define TXB0            0x31
#define TXB1            0x41
#define TXB2            0x51
#define RXB0            0x61
#define RXB1            0x71
*/

#define EXIDE_SET       0x08
#define EXIDE_RESET     0x00

#endif	// MCP2515_DEFS_H
