// ----------------------------------------------------------------------------
/*
 * Copyright (c) 2013 Lessmller Lasertechnik GmbH
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
// ----------------------------------------------------------------------------
#include "sdo_module.h"

extern unsigned char v_level;

int SDO_check_error(char *info, SDO_CMD_ *cmd)
{
		if (v_level >= 2)
		{
			printf("\r\n Reply to '%s' : ", info);
			if (cmd->flags.cs == 0x02)
			{
				printf("0x%03X|0x%02X sent %d bytes :", cmd->index, cmd->subindex, (4 - cmd->flags.n));
				for (unsigned char i = 0; i < (4 - cmd->flags.n); i++)
					printf(" 0x%02X", cmd->data[i]);
			
			} else if (cmd->flags.cs == 0x03)
			{
				printf("0x%03X|0x%02X confirms download.", cmd->index, cmd->subindex);
			} else if (cmd->flags.cs == 0x01)
			{
				printf("Acknowledge of upload.");
			} else if (cmd->flags.cs == 0x00)
			{
				printf("Downloaded %d bytes : ", (7-cmd->flags.n));
				for (unsigned char i = 0; i < (7 - cmd->flags.n); i++)
					printf(" 0x%02X", cmd->data[i]);
			} else if (cmd->flags.cs == 0x04)
			{
				printf("0x%03X|0x%02X sent SDO Abort Code.", cmd->index, cmd->subindex);

				if (cmd->data[3] == 0x0F)
				{
					switch (cmd->data[0])
					{
						case 0x01 : printf("\r\n INVALID_COMMAND"); break;
						case 0x02 : printf("\r\n SRC_ADDR_ERROR"); break;
						case 0x04 : printf("\r\n SRC_ADDR_NOT_MAPPED"); break;
						case 0x05 : printf("\r\n DST_ADDR_NOT_MAPPED"); break;
						case 0x06 : printf("\r\n COUNT_ERROR"); break;
						case 0x07 : printf("\r\n INVALID_SECTOR"); break;
						case 0x08 : printf("\r\n SECTOR_NOT_BLANK"); break;
						case 0x09 : printf("\r\n SECTOR_NOT_PREPARED_FOR_WRITE_OPERATION"); break;
						case 0x0A : printf("\r\n COMPARE_ERROR"); break;
						case 0x0C : printf("\r\n PARAM_ERROR"); break;
						case 0x0D : printf("\r\n ADDR_ERROR"); break;
						case 0x0E : printf("\r\n ADDR_NOT_MAPPED"); break;
						case 0x0F : printf("\r\n CMD_LOCKED"); break;
						case 0x10 : printf("\r\n INVALID_CODE"); break;
						case 0x13 : printf("\r\n CODE_READ_PROTECTION_ENABLED"); break;
						case 0x00 : printf("\r\n NO_ERROR"); break;
						default   : printf("\r\n UNKNOWN_ERROR_CODE");  break;
					}
					return cmd->data[0];
				} else
				{
					printf("\r\n 0x%02X 0x%02X 0x%02X 0x%02X", cmd->data[0], cmd->data[1], cmd->data[2], cmd->data[3]);
					return -1;
				}

			} else
				printf(" 0x%02X 0x%04X 0x%02X 0x%02X 0x%02X 0x%02X 0x%02X", cmd->flags, cmd->index, cmd->subindex, cmd->data[0], cmd->data[1], cmd->data[2], cmd->data[3]);
		}
	return 0;
}

/* SDO_CMD
 * 
 * This Function creates the Service Data Object (SDO) Command and sends it over CAN.
 *
 * 0. Byte: cs: Command Specifier;
 *			 n: number of data bytes which do not contain data, only if both e and s are set;
 *			 e: If 1, indicates an expedited transfer (data is encapsulated in the 4 data bytes of the CAN frame). If 0, transfer is of segmented type.
 *		     s: If 1, data size is indicated in n (e = 1) or data size is indicated in data (e = 0).
 * 1. and 2.Byte: Index;
 * 4.Byte: Subindex;
 * 5. - 7.Byte: Data
 */
int SDO_CMD(unsigned int sdo_id, SDO_CMD_ *SDO_CMD)
{
	unsigned char cmd[8];

	cmd[0] = *((unsigned char*) &(SDO_CMD->flags));
	cmd[1] = (unsigned char) (SDO_CMD->index);
	cmd[2] = (unsigned char) (SDO_CMD->index >> 8);
	cmd[3] = SDO_CMD->subindex;

	for (unsigned char i = 0; i < (4 - SDO_CMD->flags.n); i++)
		cmd[4+i] = SDO_CMD->data[i];
	for (unsigned char i = (4 - SDO_CMD->flags.n); i < 4; i++)
		cmd[4+i] = 0x00;

	return CAN_send(0, can_message_(sdo_id, 0, 0, 8, cmd));
}

/* SDO_SEG
 * 
 * This Function creates the SDO Command for segmented writing and sends it over CAN.
 *
 * 0. Byte: cs: Command Specifier,
			 t: toggle,
			 n: Indicates the number of data bytes which do not contain data
 *		     l: Last frame. Indicates for segmented transfer that this is the last frame.
 * 1. - 7.Byte: Data
 */
int SDO_SEG(unsigned int sdo_id, unsigned char cmd_specifier, unsigned char last, unsigned char size, unsigned char *data)
{
	unsigned char cmd[8];
	static unsigned char toggle = 0;
	unsigned char n;

	if (size > 7) size = 7;

	n = 7 - size;
	
	cmd[0] = (unsigned char) (((cmd_specifier & 0x07) << 5) | ((toggle & 0x01) << 4) | ((n & 0x07) << 1) | ((last & 0x01) << 0));

	for (unsigned char i = 0; i < size; i++)
		cmd[1+i] = data[i];
	for (unsigned char i = size; i < 7; i++)
		cmd[1+i] = 0x00;

	toggle ^= 0x01;

	if (last == 1)
		toggle = 0;

	return CAN_send(0, can_message_(sdo_id, 0, 0, 8, cmd));
}

/* SDO_CMD_REPLY
 * 
 * This Function receives the answer over CAN and reads the SDO Command.
 *
 * 0. Byte: cs: Command Specifier;
 *   		 n: number of data bytes which do not contain data, only if both e and s are set;
 *		     e: if 1, indicates an expedited transfer (data is encapsulated in the 4 data bytes of the CAN frame). If 0, transfer is of segmented type.
 *		     s: If 1, data size is indicated in n (e = 1) or data size is indicated in data (e = 0).
 * 1. and 2.Byte: Index;
 * 4.Byte: Subindex;
 * 5. - 7.Byte: Data
 */
int SDO_SEG_(unsigned int sdo_id, unsigned char cmd_specifier, unsigned char last, unsigned char size, ...)
{
	unsigned char data[7];

	va_list args;
	va_start(args, size);

	if (size > 7) size = 7;
	
	for (unsigned int i = 0; i < size; i++)
		data[i] = (unsigned char) va_arg(args, int);

	va_end(args);

	return SDO_SEG(sdo_id, cmd_specifier, last, size, data);
}

int SDO_CMD_REPLY(unsigned int sdo_id, SDO_CMD_ *SDO_CMD)
{
	unsigned char t = 0;
	signed char buffer;
	can_message_ reply;
	
	do
	{
		CAN_check_rx(&buffer, NULL, NULL, NULL);
		Sleep(10);
	
		switch (buffer)
		{
			case 0:
			case 1: CAN_recv(buffer, &reply); break;
			case 2: CAN_recv(0, &reply);
					if (reply.id != (sdo_id))
						CAN_recv(1, &reply);
					break;
		}
		
		t++;
	}
	while (t < SDO_REPLY_TIMEOUT && (reply.id != (sdo_id)));

	if (t == SDO_REPLY_TIMEOUT)
		return -1;
	
	SDO_CMD->flags.raw = reply.data[0];

	SDO_CMD->index = (unsigned short) (reply.data[1] + ((unsigned short) reply.data[2] << 8));
	SDO_CMD->subindex = reply.data[3];

	for (unsigned char i = 0; i < 4; i++)
		SDO_CMD->data[i] = reply.data[4+i];

	return 0;
}

/* SDO_SEG_REPLY
 * 
 * This Function receives the answer over CAN and reads the SDO Command for segmented reading.
 *
 * 0. Byte: cs: Command Specifier,
 *			 t: toggle,
 *			 n: Indicates the number of data bytes which do not contain data
 *		     l: Last frame. Indicates for segmented transfer that this is the last frame.
 * 1. - 7.Byte: Read Data
 */
int SDO_SEG_REPLY(unsigned int sdo_id, unsigned char *last, unsigned char *size, unsigned char *data)
{
	unsigned char t = 0;
	signed char buffer;
	can_message_ reply;

	do
	{
		CAN_check_rx(&buffer, NULL, NULL, NULL);
		Sleep(10);
	
		switch (buffer)
		{
			case 0:
			case 1: CAN_recv(buffer, &reply); break;
			case 2: CAN_recv(0, &reply);
					if (reply.id != (sdo_id))
						CAN_recv(1, &reply);
					break;
		}
				
		t++;
	}
	while (t < SDO_REPLY_TIMEOUT && (reply.id != (sdo_id)));

	if (t == SDO_REPLY_TIMEOUT)
		return -1;

	*last = reply.data[0] & 0x01;
	*size = (reply.data[0] & 0x0E) >> 1;

	if (*size > 7) *size = 7;
	
	for (unsigned char i = 0; i < (unsigned char) (7 - *size); i++)
		data[i] = reply.data[1+i];
	for (unsigned char i = (unsigned char) (7 - *size); i < 7; i++)
		data[i] = 0x00;

	return 0;
}
