// ----------------------------------------------------------------------------
/*
 * Copyright (c) 2013 Lessmller Lasertechnik GmbH
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
// ----------------------------------------------------------------------------
#ifndef _SDO_MODULE_H
#define _SDO_MODULE_H

#include "can_module.h"
#include "fancy_console.h"

#define SDO_REPLY_TIMEOUT 100

#define SDO_CS_SEG_DOWNLOAD		0
#define SDO_CS_INIT_DOWNLOAD	1
#define SDO_CS_INIT_UPLOAD		2
#define SDO_CS_SEG_UPLOAD		3
#define SDO_CS_ABORT_TRANSFER	4

#define SDO_TRANSFER_EXPEDITED	1
#define SDO_TRANSFER_SEGMENTED	0

#define SDO_TRANSFER_LAST		1
#define SDO_TRANSFER_NOT_LAST	0

#pragma pack(push,1)		// Pack bitfields in one byte

typedef struct SDO_CMD_
{
	union flags
	{
		struct
		{
			unsigned char s : 1;
			unsigned char e : 1;
			unsigned char n : 2;
			unsigned char reserved : 1;
			unsigned char cs : 3;
		};
		
		unsigned char raw;

		flags() : cs(0), reserved(0), n(0), e(0), s(0){};
		flags(unsigned char cs_, unsigned char n_, unsigned char e_, unsigned char s_) : cs(cs_),reserved(0), n(n_), e(e_), s(s_) {};
	} flags;

	unsigned short index;
	unsigned char subindex;
	unsigned char data[4];

	unsigned int get_data_32()
	{
		return (unsigned int) (data[0] + (data[1] << 8) +  (data[2] << 16) + (data[3] << 24));
	}

	unsigned short get_data_16()
	{
		return (unsigned short) (data[0] + (data[1] << 8));
	}


	SDO_CMD_() : flags(), index(0), subindex(0) {};

	SDO_CMD_(unsigned char command_specifier, unsigned char transfer_type, unsigned short index_, unsigned char subindex_, unsigned char size, unsigned char *data_) : index(index_), subindex(subindex_)
	{
		if (size > 4) size = 4;

		flags.cs = command_specifier;
		flags.reserved = 0;
		flags.e  = transfer_type;

		if (transfer_type == SDO_TRANSFER_EXPEDITED)
		{
			flags.n = 4 - size;
			flags.s = 1;
		}
		else if (transfer_type == SDO_TRANSFER_SEGMENTED && command_specifier == SDO_CS_INIT_DOWNLOAD)
		{
			flags.n = 0;
			flags.s = 1;
		} else
		{
			flags.n = 0;
			flags.s = 0;
		}

		for (unsigned char i = 0; i < size; i++)
			data[i] = data_[i];

	}

	SDO_CMD_(unsigned char command_specifier, unsigned char transfer_type, unsigned short index_, unsigned char subindex_, unsigned char size, ...) : index(index_), subindex(subindex_)
	{
		va_list args;
		va_start(args, size);

		if (size > 4) size = 4;

		flags.cs = command_specifier;
		flags.reserved = 0;
		flags.e  = transfer_type;

		if (transfer_type == SDO_TRANSFER_EXPEDITED)
		{
			flags.n = 4 - size;
			flags.s = 1;
		}
		else if (transfer_type == SDO_TRANSFER_SEGMENTED && command_specifier == SDO_CS_INIT_DOWNLOAD)
		{
			flags.n = 0;
			flags.s = 1;
		} else
		{
			flags.n = 0;
			flags.s = 0;
		}

		for (unsigned char i = 0; i < size; i++)
			data[i] = (unsigned char) va_arg(args, int);
		for (unsigned char i = size; i < 4; i++)
			data[i] = 0x00;

		va_end(args);
	}

} SDO_CMD_;
#pragma pack(pop)

int SDO_CMD(unsigned int sdo_id, SDO_CMD_ *SDO_CMD);

int SDO_SEG(unsigned int sdo_id, unsigned char cmd_specifier, unsigned char last, unsigned char size, unsigned char *data);
int SDO_SEG_(unsigned int sdo_id, unsigned char cmd_specifier, unsigned char last, unsigned char size, ...);

int SDO_CMD_REPLY(unsigned int sdo_id, SDO_CMD_ *SDO_CMD);
int SDO_SEG_REPLY(unsigned int sdo_id, unsigned char *last, unsigned char *size, unsigned char *data);

int SDO_check_error(char *info, SDO_CMD_ *cmd);


#endif
