// ----------------------------------------------------------------------------
/*
 * Copyright (c) 2013 Lessmller Lasertechnik GmbH
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
// ----------------------------------------------------------------------------
#include "spi_module.h"

FT_HANDLE ftHandle;

int SPI_init(void)
{
	Init_libMPSSE();
	
	if (SPI_open_channel(SPI_CHANNEL) == -1)
		return -1;											// Could not open SPI-Channel
	
	return 0;
}

int SPI_read_data(unsigned char size, unsigned char *data)
{
	unsigned long sizeTransfered = 0;
	unsigned int status;
	
	status = SPI_Read(ftHandle, data, size, &sizeTransfered, SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE);
	APP_CHECK_STATUS(status);
	
	return status;
}

int SPI_read_byte(unsigned char *data)
{
	unsigned int status;
	
	status = SPI_read_data(1, data);
	APP_CHECK_STATUS(status);
	
	return status;
}

int SPI_write_data(unsigned char size, unsigned char *data)
{
	unsigned int status;
	unsigned long sizeTransfered = 0;
	
	status = SPI_Write(ftHandle, data, size, &sizeTransfered, SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE);
	APP_CHECK_STATUS(status);
	
	return status;
}

int SPI_write_byte(unsigned char *data)
{
	unsigned int status;

	status = SPI_write_data(1, data);
	APP_CHECK_STATUS(status);
	
	return status;
}

int SPI_reset_CS(void)
{
	unsigned int status;
	unsigned long int sizeTransfered = 0;
	unsigned char dummy;

	status = SPI_Read(ftHandle, &dummy, 0, &sizeTransfered, SPI_TRANSFER_OPTIONS_SIZE_IN_BITS | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);
	APP_CHECK_STATUS(status);
	
	Sleep(1);
	
	return status;
}

int SPI_open_channel(unsigned int channel) 
{
	unsigned long int channels;
	ChannelConfig channelConf;
	unsigned int status;
	FT_DEVICE_LIST_INFO_NODE devList;
	
	printf(".");

	status = SPI_GetNumChannels(&channels);
	APP_CHECK_STATUS(status);
	
	printf(".");

	if(channels > 0)
	{
		// FT2232H Configuration
		channelConf.ClockRate = 5000;
		channelConf.LatencyTimer = 255;
		// Sample Data on the rising Edge and transmit on the falling
		// CS = ADBUS3
		channelConf.configOptions = SPI_CONFIG_OPTION_MODE0 | SPI_CONFIG_OPTION_CS_DBUS3 | SPI_CONFIG_OPTION_CS_ACTIVELOW ;	
		channelConf.Pin = 0x00000000;	//FinalVal-FinalDir-InitVal-InitDir (for dir 0=in, 1=out)
		
		printf(".");
		
		// Get SPI Channel Information
		status = SPI_GetChannelInfo(channel, &devList); 
		APP_CHECK_STATUS(status);
		
		printf(".");
		
		// Open SPI Channel
		status = SPI_OpenChannel(channel, &ftHandle);
		APP_CHECK_STATUS(status);
	
		printf(".");

		// Set Channel Configuration
		status = SPI_InitChannel(ftHandle, &channelConf); 
		APP_CHECK_STATUS(status);
		
		printf(".");

		return status;
	}
	else
	{
		return -1;
	}
}

int SPI_close_channel(void)
{
	int status = SPI_CloseChannel(ftHandle);
	Cleanup_libMPSSE();
	return status;
}
