// ----------------------------------------------------------------------------
/*
 * Copyright (c) 2013 Lessmller Lasertechnik GmbH
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
// ----------------------------------------------------------------------------

#ifndef _SPI_MODULE_
#define _SPI_MODULE_

/* SPI Communciation Module
 * 
 * This Module provides functions for transferring data over the SPI Bus.
 *
 */

/* FTDI Specific #Includes and #Defines */

#ifdef _WIN32
#include <windows.h>
#endif

#include <stdio.h>

#define SPI_CHANNEL 0

#include "libMPSSE_spi.h"

#ifdef _DEBUG
	#define APP_CHECK_STATUS(exp)	{if(exp!=FT_OK){printf("\r\n%s:%d:%s(): status(0x%x) != FT_OK\r\n",__FILE__, __LINE__, __FUNCTION__,exp);exit(1);}else{;}};
	#define CHECK_NULL(exp)			{if(exp==NULL){printf("\r\n%s:%d:%s():  NULL expression encountered \r\n",__FILE__, __LINE__, __FUNCTION__);exit(1);}else{;}};
#else
	#define APP_CHECK_STATUS(exp)	;
	#define	CHECK_NULL(exp)			;
#endif

int SPI_open_channel(unsigned int channel);		// Open SPI Communications Channel
int SPI_close_channel(void);					// Close SPI Communications Channel

/* SPI Functions */

int SPI_init(void);											// Initialize SPI Communication
int SPI_read_data(unsigned char size, unsigned char *data);	// Read a specified number of bytes from the SPI Interface
int SPI_read_byte(unsigned char *data);						// Read one byte of data from the SPI Interface
int SPI_write_data(unsigned char size, unsigned char *data);// Write a specified number of bytes to the SPI Interface
int SPI_write_byte(unsigned char *data);					// Write one byte of data to the SPI Interface
int SPI_reset_CS(void);										// Reset the ChipSelect Line of the SPI Interface

#endif